"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressServerAdapter = void 0;
const HttpResponse_1 = require("../../entities/HttpResponse");
const Server_1 = require("../../entities/Server");
const HttpRequest_1 = require("../../entities/HttpRequest");
const express_1 = __importDefault(require("express"));
require("express-async-errors");
const multer_1 = __importDefault(require("multer"));
const cors_1 = __importDefault(require("cors"));
class ExpressServerAdapter extends Server_1.Server {
    constructor(config, corsOptions) {
        super(config);
        this.app = (0, express_1.default)();
        this.app.use((0, cors_1.default)(corsOptions));
        this.app.use(express_1.default.json());
        this.app.use(express_1.default.urlencoded({ extended: true }));
        this.multer = (0, multer_1.default)({
            dest: this.config.uploadDir,
        });
    }
    parseResponse(res, response) {
        res.status(response.statusCode);
        for (const [name, value] of Object.entries(response.headers)) {
            value && res.setHeader(name, value);
        }
        const contentType = response.getHeader('content-type');
        if (contentType && contentType.toString().startsWith('application/json')) {
            return res.json(response.body);
        }
        return res.send(response.body);
    }
    parseRequest(req) {
        const query = {};
        const url = new URL(`http://localhost${req.originalUrl}`);
        let params = {};
        for (const key of url.searchParams.keys()) {
            const values = url.searchParams.getAll(key);
            query[key] = values.length > 1 ? values : values[0];
            if (this.config.mapQueryToParams) {
                params[key] = query[key];
            }
        }
        if (this.config.mapBodyToParams && typeof req.body === 'object') {
            params = { ...params, ...req.body };
        }
        params = { ...params, ...req.params };
        const request = new HttpRequest_1.HttpRequest({
            headers: req.headers,
            query,
            params,
            url: req.url,
            path: req.path,
            body: req.body,
        });
        if (req.files && Array.isArray(req.files))
            request.files = req.files;
        return request;
    }
    on(method, path, middlewares) {
        this.app[method](path, this.multer.any(), async (req, res) => {
            const request = this.parseRequest(req);
            let response = new HttpResponse_1.HttpResponse();
            try {
                for (const middleware of middlewares) {
                    const handlerResponse = await middleware(request, response, (error) => {
                        if (error)
                            throw error;
                    });
                    if (handlerResponse && handlerResponse instanceof HttpResponse_1.HttpResponse)
                        return this.parseResponse(res, handlerResponse);
                }
                return this.parseResponse(res, response);
            }
            catch (error) {
                if (error instanceof Error && this.onError) {
                    response = await this.onError(request, response, error);
                }
                return this.parseResponse(res, response);
            }
        });
    }
    listen(port, callback) {
        this.listener = this.app.listen(port, callback);
    }
    close() {
        return new Promise((resolve) => {
            this.listener?.close(() => {
                resolve();
            });
        });
    }
}
exports.ExpressServerAdapter = ExpressServerAdapter;
