"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestifyServerAdapter = void 0;
const Server_1 = require("../../entities/Server");
const restify_cors_middleware2_1 = __importDefault(require("restify-cors-middleware2"));
const restify_errors_1 = require("restify-errors");
const path_1 = require("path");
const restify_1 = require("restify");
const HttpRequest_1 = require("../../entities/HttpRequest");
const HttpResponse_1 = require("../../entities/HttpResponse");
const handlerDict = {
    get: 'get',
    post: 'post',
    patch: 'patch',
    delete: 'del',
    put: 'put',
    options: 'opts',
};
class RestifyServerAdapter extends Server_1.Server {
    constructor(config, options) {
        super(config);
        this.app = (0, restify_1.createServer)(options);
        const cors = (0, restify_cors_middleware2_1.default)(options?.corsOptions || {});
        this.app.pre(cors.preflight);
        this.app.pre(restify_1.pre.sanitizePath());
        this.app.use(cors.actual);
        this.app.use(restify_1.plugins.fullResponse());
        this.app.use(restify_1.plugins.queryParser({
            mapParams: !!this.config?.mapQueryToParams,
        }));
        this.app.use(restify_1.plugins.bodyParser({
            mapParams: !!this.config?.mapBodyToParams,
            uploadDir: this.config.uploadDir,
        }));
    }
    parseRequest(req) {
        const request = new HttpRequest_1.HttpRequest({
            headers: req.headers,
            params: req.params,
            path: req.getPath(),
            query: req.query,
            url: req.url || '',
            body: req.body,
        });
        for (const [name, file] of Object.entries(req.files || {})) {
            file &&
                request.files.push({
                    fieldname: name,
                    originalname: file.name || '',
                    filename: (0, path_1.basename)(file.path),
                    size: file.size,
                    mimetype: file.type || '',
                    path: file.path,
                });
        }
        return request;
    }
    parseResponse(res, response) {
        res.status(response.statusCode);
        for (const [name, value] of Object.entries(response.headers)) {
            value && res.setHeader(name, value);
        }
        const contentType = response.getHeader('content-type');
        if (contentType && contentType.toString().startsWith('application/json')) {
            res.json(response.body);
            return;
        }
        res.send(response.body);
    }
    on(method, path, middlewares) {
        this.app[handlerDict[method]](path, async (req, res) => {
            const request = this.parseRequest(req);
            let response = new HttpResponse_1.HttpResponse();
            try {
                for (const middleware of middlewares) {
                    const handlerResponse = await middleware(request, response, (error) => {
                        if (error)
                            throw error;
                    });
                    if (handlerResponse && handlerResponse instanceof HttpResponse_1.HttpResponse)
                        return this.parseResponse(res, handlerResponse);
                }
                return this.parseResponse(res, response);
            }
            catch (error) {
                if (error instanceof restify_errors_1.HttpError) {
                    if (!response.body) {
                        response.json(error.body);
                    }
                    response.status(error.statusCode);
                }
                if (error instanceof Error && this.onError) {
                    response = await this.onError(request, response, error);
                }
                return this.parseResponse(res, response);
            }
        });
    }
    listen(port, callback) {
        this.app.listen(port, callback);
    }
    close() {
        return new Promise((resolve) => this.app.close(resolve));
    }
}
exports.RestifyServerAdapter = RestifyServerAdapter;
